@@ -4,6 +4,7 @@ import os
 import sys
 import sqlite3
 import json
+import typing
 from pkg_resources import parse_version
 from functools import partial
 from logging import DEBUG
@@ -19,7 +20,6 @@ from cutevariant.core import get_sql_connection, get_metadatas, command
 from cutevariant.core.sql import get_database_file_name
 from cutevariant.core.writer import CsvWriter, PedWriter
 from cutevariant.gui import FIcon
-from cutevariant.gui.state import State
 from cutevariant.gui.wizards import ProjectWizard
 from cutevariant.gui.settings import SettingsDialog
 from cutevariant.gui.widgets.aboutcutevariant import AboutCutevariant
@@ -37,63 +37,111 @@ from cutevariant.gui import plugin
 
 LOGGER = cm.logger()
 
+import copy
+
+
+class StateData:
+    """A dictonnary like object which monitor which key changed
+
+    This is used to store application data and refresh plugins if it is required
+
+    """
+
+    def __init__(self, data=None):
+        self._data = data if data else {}
+        self._changed = set()
+
+    def __setitem__(self, key, value):
+
+        if key in self._data:
+            if self._data[key] == value:
+                return
+
+        self._changed.add(key)
+        self._data[key] = copy.deepcopy(value)
+
+    def __getitem__(self, key):
+        if key in self._data:
+            return self._data[key]
+        return None
+
+    def clear_changed(self):
+        self._changed.clear()
+
+    @property
+    def changed(self):
+        return self._changed
+
 
 class MainWindow(QMainWindow):
-    """Main window of Cutevariant"""
+    """Cutevariant mainwindow
 
-    variants_load_finished = Signal(int, float)
+    The mainwindow is build by with plugins loaded from /cutevariant/gui/plugins.
+    It also plays the role of a mediator between plugins where data are read and write from the state_data attributes.
+
+    Attributes:
+        conn (sqlite.Connection): the sqlite databases
+        plugins : a dictionnary of plugins with plugin name as key
+
+    """
 
     def __init__(self, parent=None):
 
         super().__init__(parent)
 
+        ## ===== CLASS ATTRIBUTES =====
+        self.conn = None
+        # Plugins
+        self.plugins = {}  # dict of plugins with plugin name as key
+        self.dialog_plugins = {}  # dict of dialog plugins
+
+        # Settings
+        self.app_settings = QSettings()
+
+        # State variable of application changed by plugins
+        self._state_data = StateData(
+            {
+                "fields": ["chr", "pos", "ref", "alt"],
+                "source": "variants",
+                "filters": {},
+            }
+        )
+
+        ## ===== GUI Setup =====
+
         self.setWindowTitle("Cutevariant")
-        self.toolbar = self.addToolBar("maintoolbar")
-        self.toolbar.setObjectName("maintoolbar")  # For window saveState
         self.setWindowIcon(QIcon(DIR_ICONS + "app.png"))
         self.setWindowFlags(Qt.WindowContextHelpButtonHint | self.windowFlags())
 
-        # Keep sqlite connection
-        self.conn = None
+        # Setup menu bar
+        self.setup_menubar()
 
-        # App settings
-        self.app_settings = QSettings()
+        # Setup ToolBar
+        self.toolbar = self.addToolBar("maintoolbar")
+        self.toolbar.setObjectName("maintoolbar")  # For window saveState
+        self.setup_toolbar()
 
-        # State variable of application
-        # store fields, source, filters, group_by, having data
-        # Often changed by plugins
-        self.state = State()
-
-        # Central workspace
+        # Setup Central widget
         self.central_tab = QTabWidget()
         self.footer_tab = QTabWidget()
         self.vsplit = QSplitter(Qt.Vertical)
         self.vsplit.addWidget(self.central_tab)
         self.vsplit.addWidget(self.footer_tab)
         self.setCentralWidget(self.vsplit)
+        self.setTabPosition(Qt.AllDockWidgetAreas, QTabWidget.North)
+        # registers plugins
+        self.register_plugins()
 
-        # Status Bar
+        # Setup status bar
         self.status_bar = QStatusBar()
         self.setStatusBar(self.status_bar)
 
-        # Setup menubar
-        self.setup_menubar()
-        # Setup toobar under the menu bar
-        self.setup_toolbar()
-
-        # Register plugins
-        self.plugins = {}  # dict of names (not titles) as keys and widgets as values
-        self.dialog_plugins = {}  # dict of actions as keys and classes as values
-        self.register_plugins()
-
         # Window geometry
         self.resize(600, 400)
         self.setGeometry(
             QApplication.instance().desktop().rect().adjusted(100, 100, -100, -100)
         )
 
-        self.setTabPosition(Qt.AllDockWidgetAreas, QTabWidget.North)
-
         # If True, the GUI settings are deleted when the app is closed
         self.requested_reset_ui = False
         # Restores the state of this mainwindow's toolbars and dockwidgets
@@ -104,8 +152,45 @@ class MainWindow(QMainWindow):
         if recent and os.path.isfile(recent[0]):
             self.open(recent[0])
 
+    def set_state_data(self, key: str, value: typing.Any):
+        """set state data value from key
+
+        This method must be called by plugins to change state_data.
+        Do not use the  protected attribut (`self._state_data`) because the following method store and detect changes
+
+        Args:
+            key (str): Name of the state variable ( fields, source, filters )
+            value (Any): a value
+        """
+        self._state_data[key] = value
+
+    def get_state_data(self, key: str) -> typing.Any:
+        """Get state data value from from key
+
+        This method must be called by plugins to reads state_data
+
+        Args:
+            key (str): Name of the state variable
+
+        Returns:
+            typing.Any: Return a value
+        """
+
+        return self._state_data[key]
+
     def add_panel(self, widget, area=Qt.LeftDockWidgetArea):
-        """Add given widget to a new QDockWidget and to view menu in menubar"""
+        """Add the given widget to a new QDockWidget.
+
+        It the most of the case, it is a PlufinWidget with PluginWidget.LOCATION equal to DOCK_LOCATION
+
+        Note:
+            Adding panel will append a hide/show action in the view menu
+
+        Args:                LOGGER.debug("Connected variant view signals to mainwindow")
+
+            widget (QWidget): a standard QWidget like a PluginWidget
+            area: Area location. Defaults to Qt.LeftDockWidgetArea.
+        """
         dock = QDockWidget()
         dock.setWindowTitle(widget.windowTitle())
         dock.setWidget(widget)
@@ -124,34 +209,28 @@ class MainWindow(QMainWindow):
         self.toolbar.addAction(action)
 
     def register_plugins(self):
-        """Dynamically load plugins to the window
+        """Load all plugins to the window
 
-        Wrapper of :meth:`register_plugin`
+        See self.register_plugin
 
-        Two types of plugins can be registered:
-        - widget: Added to the GUI,
-        - dialog: DialogBox accessible from the tool menu.
-
-        `setting` type is handled separately in :meth:`cutevariant.gui.settings.load_plugins`
         """
         LOGGER.info("MainWindow:: Registering plugins...")
-
         # Get classes of plugins
-        # Don't forget to skip disabled plugins
         for extension in plugin.find_plugins():
             self.register_plugin(extension)
 
     def register_plugin(self, extension):
-        """Dynamically load plugins to the window
+        """Load a plugin to the Mainwindow
 
-        Two types of plugins can be registered:
-        - widget: Added to the GUI,
-        - dialog: DialogBox accessible from the tool menu.
+        There are Two kinds of plugins which can be registered:
+        - PluginWidget: Added to the mainwindow as QDockWidget or TabWidget
+        - PluginDialog: DialogBox accessible from the tool menu.
 
-        `setting` type is handled separately in :meth:`cutevariant.gui.settings.load_plugins`
+        Note:
+            PluginSettingsWidget is handled separately in`cutevariant.gui.settings.load_plugins`
 
         Args:
-            extension (dict): Extension dict. See :meth:`cutevariant.gui.plugin.find_plugins`
+            extension (dict): Extension dict returned by `cutevariant.gui.plugin.find_plugins`
         """
         LOGGER.debug("Extension: %s", extension)
 
@@ -165,16 +244,7 @@ class MainWindow(QMainWindow):
 
             # Setup new widget
             widget = plugin_widget_class(parent=self)
-
-            """Variant view is a special widget that loads variants and counts them.
-            Every plugin should be warned whenever the variants get loaded, so we need to connect variant view's signals to mainwindow's signals"""
-            if name == "variant_view":
-
-                LOGGER.debug("Loading variant view")
-                widget.variants_load_finished.connect(
-                    lambda count, time: self.variants_load_finished.emit(count, time)
-                )
-                LOGGER.debug("Connected variant view signals to mainwindow")
+            widget.setDisabled(True)
 
             if not widget.objectName():
                 LOGGER.debug(
@@ -193,31 +263,27 @@ class MainWindow(QMainWindow):
                 long_description = extension.get("description")
             widget.setWhatsThis(long_description)
             # Register (launch first init on some of them)
+            widget.mainwindow = self
             widget.on_register(self)
-            if self.conn:
-                # If register occurs after a project is loaded we must set its
-                # connection attribute
-                widget.on_open_project(self.conn)
+            # if self.conn:
+            #     # If register occurs after a project is loaded we must set its
+            #     # connection attribute
+            #     widget.on_open_project(self.conn)
 
             # Init mainwindow via the constructor or on_register
-            if widget.mainwindow != self:
-                LOGGER.error(
-                    "Bad plugin implementation, <mainwindow> plugin attribute is not set."
-                )
-                widget.on_close()
-            else:
-                # Add new plugin to plugins already registered
-                self.plugins[name] = widget
 
-                # Set position on the GUI
-                if plugin_widget_class.LOCATION == plugin.DOCK_LOCATION:
-                    self.add_panel(widget)
+            # Add new plugin to plugins already registered
+            self.plugins[name] = widget
 
-                if plugin_widget_class.LOCATION == plugin.CENTRAL_LOCATION:
-                    self.central_tab.addTab(widget, widget.windowTitle())
+            # Set position on the GUI
+            if plugin_widget_class.LOCATION == plugin.DOCK_LOCATION:
+                self.add_panel(widget)
 
-                if plugin_widget_class.LOCATION == plugin.FOOTER_LOCATION:
-                    self.footer_tab.addTab(widget, widget.windowTitle())
+            if plugin_widget_class.LOCATION == plugin.CENTRAL_LOCATION:
+                self.central_tab.addTab(widget, widget.windowTitle())
+
+            if plugin_widget_class.LOCATION == plugin.FOOTER_LOCATION:
+                self.footer_tab.addTab(widget, widget.windowTitle())
 
         if "dialog" in extension and extension["dialog"].ENABLE:
             # New tool menu entry
@@ -231,20 +297,34 @@ class MainWindow(QMainWindow):
     def refresh_plugins(self, sender: plugin.PluginWidget = None):
         """Refresh all widget plugins
 
-        It doesn't refresh the sender plugin, and not visible plugins.
+        It doesn't refresh a plugin if :
+
+        - the plugin is the sender
+        - the plugin is not visible
+        - the plugin specified a class variable REFRESH_STATE_DATA = ["fields"]
 
         Args:
             sender (PluginWidget): from a plugin, you can pass "self" as argument
         """
+
         for plugin_obj in self.plugins.values():
-            if plugin_obj is not sender and (
-                plugin_obj.isVisible() or plugin_obj.REFRESH_ONLY_VISIBLE is False
-            ):
+            need_refresh = (
+                plugin_obj is not sender
+                and (plugin_obj.isVisible() or not plugin_obj.REFRESH_ONLY_VISIBLE)
+                and (set(plugin_obj.REFRESH_STATE_DATA) & self._state_data.changed)
+            )
+
+            if need_refresh:
                 try:
                     plugin_obj.on_refresh()
+                    print(plugin_obj)
+
                 except Exception as e:
                     LOGGER.exception(e)
 
+        # Clear state_changed set
+        self._state_data.clear_changed()
+
     def refresh_plugin(self, plugin_name: str):
         """Refresh a widget plugin identified by plugin_name
 
@@ -453,7 +533,7 @@ class MainWindow(QMainWindow):
         # Clear memoization cache for count_cmd
         # Clear State variable of application
         # store fields, source, filters, group_by, having data
-        self.state = State()
+        self.state = {"fields": ["chr"], "source": "variants", "filters": {}}
 
         # Load previous window state for this project (file_path being the key for the settings)
         file_path = get_database_file_name(conn)
@@ -462,6 +542,7 @@ class MainWindow(QMainWindow):
 
         for plugin_obj in self.plugins.values():
             plugin_obj.on_open_project(self.conn)
+            plugin_obj.setEnabled(True)
 
     def save_recent_project(self, path):
         """Save current project into QSettings
