@@ -1,6 +1,6 @@
 from PySide2.QtWidgets import QApplication
 
-__title__ = "Filters"
+__title__ = "Filters editor"
 __description__ = "A plugin to display filters"
 __long_description__ = QApplication.instance().translate(
     "filters_editor",
