@@ -76,8 +76,8 @@ LOGGER = cm.logger()
 class AbstractSettingsWidget(QWidget):
     """Abstract class for settings widgets
 
-    User must reimplement load() and save() 
-     
+    User must reimplement load() and save()
+
     """
 
     def __init__(self):
@@ -98,7 +98,7 @@ class AbstractSettingsWidget(QWidget):
 
     @property
     def prefix_settings(self) -> str:
-        """ Return prefix settings from section parent  
+        """Return prefix settings from section parent
         Returns:
             str
         """
@@ -288,42 +288,26 @@ class StyleSettingsWidget(AbstractSettingsWidget):
         self.setLayout(mainLayout)
 
     def save(self):
-        """Save the selected style in config
-        """
+        """Save the selected style in config"""
         # Get previous style
 
         settings = self.create_settings()
-
         old_style_name = settings.value("ui/style", cm.BASIC_STYLE)
 
         # Save style setting
         style_name = self.styles_combobox.currentText()
         if old_style_name == style_name:
             return
+
         settings.setValue("ui/style", style_name)
 
-        # Change the style on the fly
-        # TODO: Find a way to revert properly dark() palette and fill
-        # style.bright() function.
-
-        if style_name == cm.BASIC_STYLE:
-            # Bright version: Reset style
-            QApplication.instance().setStyleSheet("")
-            QApplication.instance().setPalette(QApplication.style().standardPalette())
-        else:
-            # QApplication.instance().setStyle("fusion")
-            # Apply selected style by calling on the method in style module based on its
-            # name; equivalent of style.dark(app)
-            getattr(style, style_name.lower())(QApplication.instance())
+        QMessageBox.information(
+            self, "restart", self.tr("Please restart application to apply theme")
+        )
 
         # Clear pixmap cache
         QPixmapCache.clear()
 
-        # Get the window for this widget,
-        # i.e. the next ancestor widget that has a window-system frame.
-        # And call refresh signal => reload the widgets
-        self.window().uiSettingsChanged.emit()
-
     def load(self):
         """Setup widgets in StyleSettingsWidget"""
         self.styles_combobox.clear()
@@ -455,26 +439,26 @@ class PathSettingsWidget(AbstractSettingsWidget):
 
     def __init__(self):
         super().__init__()
-        self.setWindowTitle(self.tr("Path"))
+        self.setWindowTitle(self.tr("Global settings"))
         self.setWindowIcon(FIcon(0xF1080))
 
         self.edit = widgets.FileEdit()
         self.edit.set_path_type("dir")
         main_layout = QFormLayout()
-        main_layout.addRow("path", self.edit)
+        main_layout.addRow("Preset path", self.edit)
 
         self.setLayout(main_layout)
 
     def save(self):
         settings = QSettings()
         if self.edit.exists():
-            settings.setValue("data_path", self.edit.text())
+            settings.setValue("preset_path", self.edit.text())
 
     def load(self):
 
         settings = QSettings()
         path = settings.value(
-            "data_path",
+            "preset_path",
             QStandardPaths.writableLocation(QStandardPaths.GenericDataLocation),
         )
 
